use super::position::Span;



/// An error that has occured during lexigraphical analysis.
#[derive(Debug, Clone, PartialEq)]
pub struct LexerError
{
   /// A human-readable error message.
   pub message: String,

   /// The start and end of where the error is located in the file.
   pub span: Span,

   /// The file that the error occured within.
   pub file: Option<std::path::PathBuf>,

   /// The problematic string (optional).
   pub snippet: Option<String>
}



impl LexerError
{
   pub fn new<S, T>(message: S, span: Span, file: Option<T>,
                    snippet: Option<S>)
                    -> Self
      where S: Into<String>,
            T: Into<std::path::PathBuf>
   {
      LexerError { message: message.into(),
                   span,
                   file: file.map(|t| t.into()),
                   snippet: snippet.map(|s| s.into()) }
   }
}

impl std::fmt::Display for LexerError
{
   fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result
   {
      match &self.snippet
      {
         Some(snippet) =>
         {
            write!(f,
                   "LexerError at {}: {} (snippet: '{}')",
                   self.span, self.message, snippet)
         }
         None =>
         {
            write!(f, "LexerError at {}: {}", self.span, self.message)
         }
      }
   }
}

impl std::error::Error for LexerError {}
